<?php
	/**
	 * @since: 11/04/2019
	 * @author: Sarwar Hasan
	 * @version 1.0.0
	 */
	
	class PaddleIntegrator extends APBDLicIntegrator {
		function initialize() {
			$this->AddAjaxAction( "view-details", [ $this, 'viewLicenseDetails' ] );
			$this->AddAjaxAction( "add-link", [ $this, 'AddNewLink' ] );
			$this->AddAjaxAction( "delete-linked", [ $this, 'DeleteLinked' ] );
			$this->AddAjaxAction( "get-links", [ $this, 'getLinks' ] );
			$this->AddAjaxAction( "save-api", [ $this, 'updateLicenseType' ] );
			$this->AddAjaxAction( "get-envato-products", [ $this, 'getEnvatoProductsKeyValue' ] );
			$this->AddAjaxAction( "get-linked-product", [ $this, 'getLinkedProductAjax' ] );
			$this->AddAjaxAction( "get-license-type", [ $this, 'get_license_type' ] );
			$this->AddAjaxAction( "delete-linked-item", [ $this, 'delete_linked_item' ] );
			Mapbd_license::RegisterMarkets( "P", "Paddle" );
			
			add_action( $this->integratorModule->getHookActionStr( "register-rest-api" ), [ $this, 'RouteRegister' ] );
			add_action( $this->integratorModule->getHookActionStr( "view-license-details-extra" ), [
				$this,
				'licenseDetailsExtra'
			] );
			$this->title = ' ';
			add_filter( 'elite-licenser/paddle-linked-product', [ $this, "getLinkedProductList" ] );
			//  delete_option('app_elpdl_test_linked');
		}
		
		public function getLinkedProductList( $linked = [] ) {
			if ( $this->GetOption( 'paddle_api_test_mode', 'N' ) == "Y" ) {
				$db_products = get_option( "app_elpdl_test_linked", [] );
			} else {
				$db_products = get_option( "app_elpdl_linked", [] );
			}
			foreach ( $db_products as $key => $db_product ) {
				$linked[ $key ] = $db_product;
			}
			
			return $linked;
		}
		
		public function AddOrUpdateLinkedProduct( $linked_object ) {
			if ( ! empty( $linked_object->paddle_id ) && ! empty( $linked_object->product_id ) && ! empty( $linked_object->license_id ) ) {
				if ( $this->GetOption( 'paddle_api_test_mode', 'N' ) == "Y" ) {
					$linkedProduct                              = get_option( "app_elpdl_test_linked", [] );
					$linkedProduct[ $linked_object->paddle_id ] = $linked_object;
					
					return update_option( "app_elpdl_test_linked", $linkedProduct ) or add_option( "app_elpdl_test_linked", $linkedProduct );
				} else {
					$linkedProduct                              = get_option( "app_elpdl_linked", [] );
					$linkedProduct[ $linked_object->paddle_id ] = $linked_object;
					
					return update_option( "app_elpdl_linked", $linkedProduct ) or add_option( "app_elpdl_linked", $linkedProduct );
				}
			} else {
				$this->AddError( "Linked product object does not have proper property" );
			}
			
			return false;
		}
		
		public function DeleteLinked() {
			$param        = APBD_GetValue( "id" );
			$mainResponse = new AppsbdAjaxConfirmResponse();
			if ( empty( $param ) ) {
				$mainResponse->DisplayWithResponse( false, __( "Invalid Request" ) );
				
				return;
			}
			$linkedProduct = $this->getLinkedProductList();
			
			if ( isset( $linkedProduct[ $param ] ) ) {
				unset( $linkedProduct[ $param ] );
				$isOk = true;
				if ( $this->GetOption( 'paddle_api_test_mode', 'N' ) == "Y" ) {
					if ( ! update_option( "app_elpdl_test_linked", $linkedProduct ) or add_option( "app_elpdl_test_linked", $linkedProduct ) ) {
						$isOk = false;
					}
				} else {
					if ( ! update_option( "app_elpdl_linked", $linkedProduct ) or add_option( "app_elpdl_linked", $linkedProduct ) ) {
						$isOk = false;
					}
				}
				if ( $isOk ) {
					$mainResponse->DisplayWithResponse( true, $this->__( "Deleted Successfully" ) );
				} else {
					$mainResponse->DisplayWithResponse( false, $this->__( "Delete failed try again" ) );
				}
			} else {
				$mainResponse->DisplayWithResponse( false, $this->__( "Delete failed try again" ) );
			}
		}
		
		public function getLinks() {
			$isForceReloadProduct = APBD_PostValue( "rfproduct", 'N' ) == "Y";
			$this->SetTitle( "Link With Paddle Product" );
			$this->SetPOPUPColClass( 'col-sm-6' );
			if ( APPSBD_IsPostBack ) {
				$this->AddWarning( print_r( $_POST, true ) );
			}
			$this->AddViewData( "linked_products", apply_filters( 'elite-licenser/paddle-linked-product', [] ) );
			$this->Display( 'getlinks' );
			exit;
		}
		
		function AddPaddleLog( $msg ) {
			if ( ! is_string( $msg ) ) {
				$msg = print_r( $msg, true );
			}
			do_action( "elite-licenser/paddle/log", $msg . "\n" );
		}
		
		function valid_api_key( $type = "" ) {
			$api_key = APBD_PostValue( "api_key", "" );
			
			return apply_filters( 'elite-licenser/api-key/verify', true, $api_key, $type );
		}
		
		function RouteRegister( $licensor ) {
			register_rest_route( $licensor, '/paddle/license/add', array(
				'methods'             => 'POST',
				'callback'            => [ $this, "paddle_license_add" ],
				'permission_callback' => '__return_true'
			) );
			register_rest_route( $licensor, '/paddle/web-hook', array(
				'methods'             => 'POST',
				'callback'            => [ $this, "paddle_web_hook" ],
				'permission_callback' => '__return_true'
			) );
			
		}
		
		/**
		 * @param $checkout_id
		 * @param $subscription
		 *
		 * @return Mapbd_license[]
		 */
		function getLicenseObject( $checkout_id, $subscription ) {
			$licobjs = NULL;
			if ( ! empty( $subscription ) ) {
				$licobjs = Mapbd_license::FindAllBy( "extra_param", $subscription );
			}
			if ( empty( $licobj ) ) {
				$licobjs = Mapbd_license::FindAllBy( "extra_param2", $checkout_id );
			}
			
			return $licobjs;
		}
		
		function paddle_web_hook() {
			$this->AddPaddleLog( "\npaddle web hook\n----------------------------\n" );
			if ( $this->verify_request( $_POST ) ) {
				$this->AddPaddleLog( "valid webhook" );
				$alert_name  = trim( sanitize_text_field( APBD_PostValue( "alert_name" ) ) );
				$checkout_id = APBD_PostValue( "checkout_id" );
				$email       = APBD_PostValue( "email" );
				if ( $alert_name == 'subscription_cancelled' ) {
					$next_bill_date = APBD_PostValue( "cancellation_effective_date" );
				} else {
					$next_bill_date = APBD_PostValue( "next_bill_date" );
				}
				$status          = trim( APBD_PostValue( "status" ) );
				$subscription_id = APBD_PostValue( "subscription_id" );
				if ( in_array( $alert_name, [
					'subscription_created',
					'subscription_updated',
					'subscription_payment_succeeded',
					'subscription_cancelled',
					'payment_refunded',
					//'payment_succeeded',
					'subscription_payment_refunded'
				] ) ) {
					$this->AddPaddleLog( "Known alert type" );
					Mapbd_paddle_log::AddPaddleWebhookFromPost();
					$licenseObjs = $this->getLicenseObject( $checkout_id, $subscription_id );
					if ( ! empty( $licenseObjs ) ) {
						$next_bill_date = date( 'Y-m-d', strtotime( $next_bill_date ) );
						foreach ( $licenseObjs as $licenseObj ) {
							$upObject = new Mapbd_license();
							if ( $alert_name == 'payment_refunded' ) {
								$refund_type = APBD_PostValue( "refund_type" );
								if($refund_type=='full'){
									$upObject->status( 'R' );
								}else{
									$upObject->status( 'I' );
								}
							} else {
								
								$upObject->has_expiry( 'Y' );
								if ( ! empty( $next_bill_date ) ) {
									$upObject->expiry_time( $next_bill_date );
								}
								
								if ( $this->GetOption( "paddle_sus", "B" ) == "B" ) {
									$upObject->support_end_time( $next_bill_date );
								}
								
								if ( $alert_name == 'subscription_cancelled' ) {
									if ( strtotime( $next_bill_date ) <= time() ) {
										$upObject->status( 'I' );
									}
								}
								if ( $alert_name == 'subscription_payment_refunded' ) {
									$refund_type       = APBD_PostValue( "refund_type" );
									$refund_status       = APBD_PostValue( "status" );
									if($refund_status != "active") {
										$expiry = date( 'Y-m-d' );
										$upObject->expiry_time( $expiry );
										$upObject->support_end_time( $expiry );
										$upObject->status( 'R' );
									}
								}
							}
							if ( $licenseObj->extra_param != $subscription_id ) {
								$upObject->extra_param( $subscription_id );
							}
							$upObject->SetWhereUpdate( "id", $licenseObj->id );
							$this->AddPaddleLog( $upObject );
							if ( $upObject->Update() ) {
								if ( $alert_name == 'subscription_cancelled' ) {
									$this->AddPaddleLog( "subscription canceled successfully" );
								} else {
									$this->AddPaddleLog( "subscription updated successfully" );
								}
							} else {
								$this->AddPaddleLog( "subscription updated failed : id " . $licenseObj->id );
							}
						}
					} else {
						$this->paddle_api_based_license_entry();
					}
				} elseif ( in_array( $alert_name, [ 'subscription_cancelled' ] ) ) {
					$this->AddPaddleLog( $_POST );
				}elseif ( in_array( $alert_name, [ 'payment_succeeded' ] ) ) {
					$this->AddPaddleLog( $_POST );
					$licenseObjs = $this->getLicenseObject( $checkout_id, $subscription_id );
					if(empty($licenseObjs)) {
						$this->paddle_api_based_license_entry();
					}
					return "Payment Succeeded Proceed by Elite Licenser";
				}elseif ( in_array( $alert_name, [ 'locker_processed' ] ) ) {
					$checkout_id = APBD_PostValue( "checkout_id" );
					if(!empty($license_key) && !empty($checkout_id)) {
						$license_key = APBD_PostValue( "licence" );
						$licenses=Mapbd_license::FindAllBy( "purchase_key", $license_key );
						foreach ( $licenses as $licens ) {
							if($licens->extra_param2!=$checkout_id) {
								$n = new Mapbd_license();
								$n->extra_param2( $checkout_id );
								$n->SetWhereUpdate("id",$licens->id);
								$n->Update();
							}
						}
					}
					return "Locker Processed by Elite Licenser";
				}
				
			}
		}
		
		function getLinkedProduct( $paddle_product_id ) {
			$linked_products = apply_filters( 'elite-licenser/paddle-linked-product', [] );
			$this->AddPaddleLog( $linked_products );
			foreach ( $linked_products as $linked_product ) {
				if ( $linked_product->paddle_id == $paddle_product_id ) {
					return $linked_product;
				}
			}
			
			return NULL;
		}
		
		function paddle_api_based_license_entry() {
			if ( $this->GetOption( 'paddle_api_status', 'I' ) == "A" ) {
				$this->AddPaddleLog( "API BASED LICENSE Enabled" );
				$alert_name = trim( sanitize_text_field( APBD_PostValue( "alert_name" ) ) );
				if ( in_array( $alert_name, [ 'subscription_created', 'payment_succeeded' ] ) ) {
					$paddle_product_id = APBD_PostValue( "product_id", '' );
					if ( empty( $paddle_product_id ) ) {
						$paddle_product_id = APBD_PostValue( "subscription_plan_id", '' );
					}
					if ( ! empty( $paddle_product_id ) ) {
						$linked_product = $this->getLinkedProduct( $paddle_product_id );
						if ( ! empty( $linked_product ) && ! empty( $linked_product->product_id ) && $linked_product->license_id ) {
							$postData                    = [];
							$postData['product_id']      = $linked_product->product_id;
							$postData['product_base']    = '';
							$postData['license_type_id'] = $linked_product->license_id;
							$postData['client_company']  = APBD_PostValue( "client_company" );
							$postData['client_email']    = APBD_PostValue( "email", "" );
							$postData['client_name']     = APBD_PostValue( "customer_name", NULL );
							$postData['client_country']  = APBD_PostValue( "country" );
							$postData['market']          = APBD_PostValue( "market", "P" );
							$postData['qty']             = APBD_PostValue( "quantity", 1 );
							$postData['extra_param2']    = APBD_PostValue( "checkout_id", "" );
							$postData['extra_param']     = APBD_PostValue( "subscription_id", "" );
							$nextBillDate                = APBD_PostValue( "next_bill_date", "" );
							if ( ! empty( $nextBillDate ) ) {
								$postData['expiry_time'] = $nextBillDate;
							}
							$postData['return_license_only'] = 1;
							$postData['seperator']           = "\n";
							$this->process_add_license( $postData );
						} else {
							$this->AddPaddleLog( "No linked product found" );
						}
					} else {
						$this->AddPaddleLog( "Product id is empty" );
					}
				}
			}else{
				$this->AddPaddleLog( "API BASED LICENSE Disabled" );
			}
			
			
		}
		
		function paddle_license_add() {
			$this->AddPaddleLog( "\npaddle web fulfillment\n----------------------------\n" );
			if ( $this->valid_api_key( "add_license" ) ) {
				if ( $this->verify_request( $_POST ) ) {
					$this->AddPaddleLog( "Request verification success" );
					$postData                    = [];
					$postData['client_name']     = APBD_PostValue( "client_name", NULL );
					$postData['client_email']    = APBD_PostValue( "client_email", "" );
					$postData['client_country']  = APBD_PostValue( "p_country" );
					$postData['client_company']  = APBD_PostValue( "client_company" );
					$postData['product_id']      = APBD_PostValue( "product_id" );
					$postData['product_base']    = APBD_PostValue( "product_base" );
					$postData['license_type_id'] = APBD_PostValue( "license_id" );
					$postData['market']          = APBD_PostValue( "market", "P" );
					$postData['extra_param2']    = APBD_PostValue( "checkout_hash", "" );
					if ( empty( $postData['extra_param2'] ) ) {
						$postData['extra_param2'] = APBD_PostValue( "extra_param", "" );
					}
					$postData['qty']         = APBD_PostValue( "p_quantity", 1 );
					$postData['extra_param'] = "";
					if ( ! empty( $postData['extra_param2'] ) ) {
						$log = Mapbd_paddle_log::FindBy( "checkout_id", $postData['extra_param2'] );
						if ( ! empty( $log ) ) {
							$postData['extra_param'] = $log->subscription_id;
							if ( ! empty( $log->next_bill_date ) ) {
								$postData['expiry_time'] = $log->next_bill_date;
							}
						}
						
					}
					if ( ! empty( $postData['extra_param2'] ) ) {
						$license_list = Mapbd_license::FindAllBy( "extra_param2", $postData['extra_param2'], [ "product_id" => $postData['product_id'] ] );
						
						if ( ! empty( $license_list ) && is_array( $license_list ) ) {
							$response = "";
							foreach ( $license_list as $licenseObj ) {
								if ( ! empty( $licenseObj->purchase_key ) ) {
									$response .= $licenseObj->purchase_key . "\n";
								}
							}
							if ( ! empty( $response ) ) {
								$this->AddPaddleLog( "Already create for this transaction" );
								$this->AddPaddleLog( $response );
								echo $response;
								exit;
							}
						}
					}
					
					$postData['return_license_only'] = 1;
					$postData['seperator']           = "\n";
					$this->AddPaddleLog( $postData );
					$this->process_add_license( $postData );
					
				} else {
					$this->AddPaddleLog( "Request verification failed" );
					echo "Request verification failed";
					die;
				}
			}
			
		}
		
		function process_add_license( $postData ) {
			if ( ! empty( $postData['license_type_id'] ) ) {
				$license_type_obj = Mapbd_license_type::FindBy( "id", $postData['license_type_id'] );
				if ( ! empty( $license_type_obj ) && $license_type_obj->has_support == "Y" ) {
					if ( isset( $postData['expiry_time'] ) ) {
						unset( $postData['expiry_time'] );
					}
				}
			}
			do_action( 'elite-licenser/license/add', $postData );
		}
		
		function verify_request( $request_data ) {
			if ( $this->GetOption( "paddle_auth_type", "ip" ) == "pb" ) {
				return $this->is_valid_public_key( $request_data );
			} else {
				return $this->is_valid_request_ip();
			}
		}
		
		function is_valid_request_ip() {
			$this->AddPaddleLog( "IP Based request checking" );
			$production_ips = [ '34.232.58.13', '34.195.105.136', '34.237.3.244' ];
			$sandbox_ips    = [ '34.194.127.46', '54.234.237.108', '3.208.120.145' ];
			$ip             = APBD_get_remote_ip();
			
			return in_array( $ip, $production_ips ) || in_array( $ip, $sandbox_ips );
		}
		
		function is_valid_public_key( $request_data ) {
			$this->AddPaddleLog( "public key based request checking" );
			$public_key_string = trim( $this->GetOption( "paddle_public_key", "" ) );
			if ( empty( $public_key_string ) ) {
				return false;
			}
			$public_key = openssl_get_publickey( $public_key_string );
			
			// Get the p_signature parameter & base64 decode it.
			$signature = base64_decode( $request_data['p_signature'] );
			
			// Get the fields sent in the request, and remove the p_signature parameter
			$fields = $request_data;
			unset( $fields['p_signature'] );
			
			// ksort() and serialize the fields
			ksort( $fields );
			foreach ( $fields as $k => $v ) {
				if ( ! in_array( gettype( $v ), array( 'object', 'array' ) ) ) {
					$fields[ $k ] = "$v";
				}
			}
			$data = serialize( $fields );
			
			// Verify the signature
			$verification = openssl_verify( $data, $signature, $public_key, OPENSSL_ALGO_SHA1 );
			
			if ( $verification == 1 ) {
				return true;
			} else {
				return false;
			}
		}
		
		function getTitleClass() {
			return 'paddle-spring-int';
		}
		
		private function getAPIandProduct() {
			$products           = Mapbd_product::FetchAllKeyValue( "id", "product_name" );
			$productLicenseList = Mapbd_product_license::FetchAll();
			$licenses           = Mapbd_license_type::FetchAllKeyValue( "id", "license_title", false, "product_id", "ASC" );
			$productLicenses    = [];
			foreach ( $productLicenseList as $productLicense ) {
				$productTitle                                                                   = isset( $products[ $productLicense->product_id ] ) ? $products[ $productLicense->product_id ] : "Unknown Product";
				$LicenseTypeTitle                                                               = isset( $licenses[ $productLicense->license_id ] ) ? $licenses[ $productLicense->license_id ] : "Unknown Product";
				$productLicenses["{$productLicense->product_id}-{$productLicense->license_id}"] = "{$productTitle}-{$LicenseTypeTitle}";
			}
			$obj           = new stdClass();
			$obj->apikeys  = Mapbd_apikey::getActiveApiKeys();
			$obj->products = $productLicenses;
			
			return $obj;
		}
		
		public function OptionForm() {
			//echo $this->getViewPath();
			$this->AddViewData( "paddleInfo", $this->getAPIandProduct() );
			$this->Display();
		}
		
		public function AddNewLink() {
			$isForceReloadProduct = APBD_PostValue( "rfproduct", 'N' ) == "Y";
			$this->SetTitle( "Link With Paddle Product" );
			$this->SetPOPUPColClass( 'col-sm-6' );
			$paddleProduct = $this->getPaddleProduct( $isForceReloadProduct );
			if ( APPSBD_IsPostBack ) {
				$selected_paddle_product        = APBD_PostValue( "paddle_product_id" );
				$selected_elite_product_license = APBD_PostValue( "el_product_id" );
				$isOk                           = true;
				if ( empty( $selected_elite_product_license ) ) {
					$this->AddError( "Paddle product selection is required" );
					$isOk = false;
				}
				if ( empty( $selected_elite_product_license ) ) {
					$this->AddError( "Elite product selection is required" );
					$isOk = false;
				}
				if ( $isOk ) {
					if ( ! empty( $paddleProduct->products[ $selected_paddle_product ] ) || ! empty( $paddleProduct->subscriptions[ $selected_paddle_product ] ) ) {
						$linked_product = $this->getLinkedProductList();
						if ( ! empty( $linked_product[ $selected_paddle_product ] ) ) {
							$this->AddError( "This paddle product is already linked" );
						} else {
							$elproduct = explode( '|', $selected_elite_product_license );
							if ( ! empty( $elproduct ) && is_array( $elproduct ) && count( $elproduct ) == 2 ) {
								if ( ! empty( $paddleProduct->products[ $selected_paddle_product ] ) ) {
									$newLinkedProduct = $paddleProduct->products[ $selected_paddle_product ];
								} else {
									$newLinkedProduct = $paddleProduct->subscriptions[ $selected_paddle_product ];
								}
								$newLinkedProduct->product_id = $elproduct[0];
								$newLinkedProduct->license_id = $elproduct[1];
								if ( $this->AddOrUpdateLinkedProduct( $newLinkedProduct ) ) {
									$this->AddInfo( "Product successfully linked" );
									$this->DisplayPOPUPMsg();
								} else {
									$this->AddError( "Save failed. Please try again" );
								}
							} else {
								$this->AddError( "Something is wrong in Elite Licenser product selection, Please try again" );
							}
						}
					} else {
						$this->AddError( "Something is wrong in paddle product selection, Please try again" );
					}
				}
			}
			$this->AddViewData( "paddle_products", $paddleProduct );
			$this->DisplayPOPUp( 'addnewlink' );
		}
		
		function getPaddleProduct( $isForceReload = false ) {
			
			$response                = new stdClass();
			$response->products      = [];
			$response->subscriptions = [];
			
			if ( $this->GetOption( 'paddle_api_test_mode', 'N' ) == "Y" ) {
				$url           = "https://sandbox-vendors.paddle.com/api/2.0/product/get_products";
				$plan_url      = "https://sandbox-vendors.paddle.com/api/2.0/subscription/plans";
				$vendor_id     = $this->GetOption( 'paddle_test_vendor_id', '' );
				$api_key       = $this->GetOption( 'paddle_test_api_Key', '' );
				$trangient_key = hash( 'crc32b', 'ptest_' . $vendor_id . '_' . $api_key );
			} else {
				$url           = "https://vendors.paddle.com/api/2.0/product/get_products";
				$plan_url      = "https://vendors.paddle.com/api/2.0/subscription/plans";
				$vendor_id     = $this->GetOption( 'paddle_vendor_id', '' );
				$api_key       = $this->GetOption( 'paddle_api_Key', '' );
				$trangient_key = hash( 'crc32b', 'ptest_' . $vendor_id . '_' . $api_key );
			}
			if ( ! $isForceReload ) {
				$json_data = get_transient( $trangient_key );
				if ( ! empty( $json_data ) ) {
					$cacheObj                = json_decode( $json_data );
					$cacheObj->products      = (array) $cacheObj->products;
					$cacheObj->subscriptions = (array) $cacheObj->subscriptions;
					
					return $cacheObj;
				}
			}
			$rq_params = [
				'method'      => 'POST',
				'sslverify'   => true,
				'timeout'     => 120,
				'redirection' => 5,
				'httpversion' => '1.0',
				'blocking'    => true,
				'headers'     => [],
				'body'        => "vendor_id=$vendor_id&vendor_auth_code=$api_key",
				'cookies'     => []
			];
			
			$serverResponse = wp_remote_post( $url, $rq_params );
			$json           = '';
			if ( is_wp_error( $serverResponse ) ) {
				$rq_params['sslverify'] = false;
				$serverResponse         = wp_remote_post( $url, $rq_params );
				if ( ! is_wp_error( $serverResponse ) ) {
					$json = wp_remote_retrieve_body( $serverResponse );
				} else {
				
				}
			} else {
				
				$json = wp_remote_retrieve_body( $serverResponse );
				
			}
			
			if ( ! empty( $json ) ) {
				$data = json_decode( $json );
				
				
				if ( ! empty( $data->success ) ) {
					//now plans
					if ( ! empty( $data->response->total ) && ! empty( $data->response->products ) ) {
						foreach ( $data->response->products as $product ) {
							$item                                   = new stdClass();
							$item->paddle_id                        = $product->id;
							$item->name                             = $product->name;
							$item->type                             = "P";
							$response->products[ $item->paddle_id ] = $item;
						}
					}
					
					$plan_response = wp_remote_post( $plan_url, $rq_params );
					if ( ! is_wp_error( $plan_response ) ) {
						$plan_json = wp_remote_retrieve_body( $plan_response );
						$data2     = json_decode( $plan_json );
						if ( ! empty( $data2->success ) ) {
							if ( ! empty( $data2->response ) ) {
								foreach ( $data2->response as $subscription ) {
									$item                                        = new stdClass();
									$item->paddle_id                             = $subscription->id;
									$item->name                                  = $subscription->name;
									$item->type                                  = "S";
									$response->subscriptions[ $item->paddle_id ] = $item;
								}
							}
						}
					}
				}
			}
			if ( ! empty( $response->products ) || ! empty( $response->subscription ) ) {
				set_transient( $trangient_key, json_encode( $response ), 10 * MINUTE_IN_SECONDS );
			}
			
			return $response;
		}
		
		/**
		 * @param Mapbd_license $mainObj
		 */
		public function licenseDetailsExtra( $mainObj ) {
			if ( strtolower( $mainObj->market ) == "p" ) {
				$this->AddViewData( "mainObj", $mainObj );
				$this->Display( 'license_details' );
			}
		}
		
	}