<?php 			
/** 
 * @since: 24/Feb/2019
 * @author: Sarwar Hasan 
 * @version 1.0.0
 * @property:id,host,ip,status,req_counter		
 */						
class Mapbd_ban_host extends AppsBDModel {
	public $id;
	public $host;
	public $ip;
	public $status;
	public $req_counter;


	/**
	 * @property id,host,ip,status,req_counter
	 */
	function __construct() {
		parent::__construct();
		$this->SetValidation();
		$this->tableName     = "apbd_ban_host";
		$this->primaryKey    = "id";
		$this->uniqueKey     = array( array( "host", "ip" ) );
		$this->multiKey      = array();
		$this->autoIncField  = array( "id" );
		$this->app_base_name = "APBLIC";
	}


	function SetValidation() {
		$this->validations = array(
			"id"          => array( "Text" => "Id", "Rule" => "xss_clean|max_length[11]|integer" ),
			"host"        => array( "Text" => "Host", "Rule" => "xss_clean|required|max_length[200]" ),
			"ip"          => array( "Text" => "Ip", "Rule" => "xss_clean|required|max_length[64]" ),
			"status"      => array( "Text" => "Status", "Rule" => "xss_clean|max_length[1]" ),
			"req_counter" => array( "Text" => "Req Counter", "Rule" => "xss_clean|required|max_length[3]|integer" )

		);
	}

	
	public function GetPropertyRawOptions($property,$isWithSelect=false){
		$returnObj=array();
		switch ($property) {
			case "status":
				$returnObj = array( "N" => "Normal", "B" => "Banned" );
				break;
			default:
		}
		if($isWithSelect){
			return array_merge(array(""=>"Select"),$returnObj);
		}
		return $returnObj;

	}
	


	
	public function GetPropertyOptionsColor($property){
		$returnObj=array();
		switch ($property) {
			case "status":
				$returnObj = array( "N" => "success", "B" => "Danger" );
				break;
		  default:
		}
		return $returnObj;

	}
	


	/*
	public function GetPropertyOptionsIcon($property){
		$returnObj=array();
		switch ($property) {
		  default:
		}
		return $returnObj;

	}
	*/


	//auto generated
	/*function Save(){
		return parent::Save();
	}*/


	static function CreateDBTable(){
		$thisObj=new static();
		$table=$thisObj->db->prefix.$thisObj->tableName;
		if($thisObj->db->get_var("show tables like '{$table}'") != $table){
			$sql = "CREATE TABLE `{$table}` (
					  `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
					  `host` char(200) CHARACTER SET latin1 COLLATE latin1_swedish_ci NOT NULL DEFAULT '',
					  `ip` char(64) CHARACTER SET latin1 COLLATE latin1_swedish_ci NOT NULL,
					  `status` char(1) CHARACTER SET latin1 COLLATE latin1_swedish_ci NOT NULL DEFAULT 'B' COMMENT 'radio(N=Normal,B=Band,)',
					  `req_counter` int(3) NOT NULL,
					  PRIMARY KEY (`id`) USING BTREE,
					  UNIQUE KEY `host` (`host`,`ip`) USING BTREE
					) ";
			require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
			dbDelta($sql);
		}
	}

	function DropDBTable() {
		global $wpdb;
		$table      = $wpdb->prefix . $this->tableName;
		$table_name = $wpdb->prefix . $this->tableName;
		$sql        = "DROP TABLE IF EXISTS $table_name;";
		$wpdb->query( $sql );
	}

	/* add custom function here*/
	static function isBanHost( $ip, $host ) {
		$host = APBD_GetUrlToHost( $host );
		$ip   = trim( $ip );
		$obj  = self::FindBy( "ip", $ip, [ "host" => $host ] );
		$isBan= ! empty( $obj->status ) && $obj->status == "B";
		$isBan=apply_filters('elite-licenser/filter/is-ban-host',$isBan,$ip,$host);
		return $isBan;
	}

	static function increaseCounter( $ip, $host ) {
		$host    = APBD_GetUrlToHost( $host );
		$ip      = trim( $ip );
		$old_obj = self::FindBy( "ip", $ip, [ "host" => $host ] );
		if ( ! empty( $old_obj ) ) {
			//update
			$obj = new self();
			$obj->req_counter( 'req_counter + 1', true );
			//ban_req_limit
			$banlimit = APBDLicenseSettings::GetModuleOption( "ban_req_limit" );
			if ( $banlimit > 0 && $banlimit < ( $old_obj->req_counter + 1 ) ) {
				$obj->status( 'B' );
			} else {
				$obj->status( 'N' );
			}
			$obj->SetWhereUpdate( "ip", $ip );
			$obj->SetWhereUpdate( "host", $host );
			$obj=apply_filters('elite-licenser/filter/update-ban-host',$obj,$ip,$host);
			if ( $obj->Update() ) {
			    do_action('elite-licenser/filter/updated-ban-host',$obj,$ip,$host);
				return true;
			}
		} else {
			$obj = new self();
			$obj->ip( $ip );
			$obj->host( $host );
			$obj->req_counter( 1 );
			$obj->status( 'N' );
			$obj=apply_filters('elite-licenser/filter/add-ban-host',$obj,$ip,$host);
			if ( $obj->Save() ) {
				do_action('elite-licenser/filter/updated-ban-host',$obj,$ip,$host);
				return true;
			}
		}
	}

	static function resetHostIpHost( $ip, $host ) {
		$host = APBD_GetUrlToHost( $host );
		$obj=new self();
		$obj->status("N");
		$obj->req_counter(0);
		$obj->SetWhereUpdate("host",$host);
		$obj->SetWhereUpdate("ip",$ip);
		return $obj->Update();
	}
	static function resetHost( $id ) {
	    $obj=new self();
	    $obj->status("N");
	    $obj->req_counter(0);
	    $obj->SetWhereUpdate("id",$id);
	    return $obj->Update();
	}

	/* end custom function */
	function GetAddForm($label_col=5,$input_col=7,$mainobj=null,$except=array(),$disabled=array()){

		if(!$mainobj){
			$mainobj=$this;
		}
		?>
        <div class="form-row">
			<?php if(!in_array("host",$except)){ ?>
                <div class="form-group col-sm">
                    <label class="col-form-label" for="host"><?php $this->_ee("Host"); ?></label>
                    <input  class="form-control form-control-sm" type="text" maxlength="200"   value="<?php echo  $mainobj->GetPostValue("host");?>" id="host" <?php echo in_array("host", $disabled)?' disabled="disabled" ':' name="host" ';?>     placeholder="<?php $this->_ee("Host");?>" data-bv-notempty="true" 	data-bv-notempty-message="<?php   $this->_ee("%s is required","Host");?>">
					
                </div>
			<?php } ?>
        </div>
        <div class="form-row">
			<?php /*if(!in_array("id",$except)){ ?>
			 <div class="form-group col-sm">
		      	<label class="col-form-label" for="id"><?php $this->_ee("Id"); ?></label>
		      	 <input  class="form-control form-control-sm" type="text" maxlength="11"   value="<?php echo  $mainobj->GetPostValue("id");?>" id="id" <?php echo in_array("id", $disabled)?' disabled="disabled" ':' name="id" ';?>     placeholder="<?php $this->_ee("Id");?>" data-bv-notempty="true" 	data-bv-notempty-message="<?php   $this->_ee("%s is required","Id");?>">

		      </div>
		     <?php } */?>

			<?php if(!in_array("ip",$except)){ ?>
                <div class="form-group col-sm">
                    <label class="col-form-label" for="ip"><?php $this->_ee("Ip"); ?></label>
                    <input  class="form-control form-control-sm" type="text" maxlength="45"   value="<?php echo  $mainobj->GetPostValue("ip");?>" id="ip" <?php echo in_array("ip", $disabled)?' disabled="disabled" ':' name="ip" ';?>     placeholder="<?php $this->_ee("Ip");?>" data-bv-notempty="true" 	data-bv-notempty-message="<?php   $this->_ee("%s is required","Ip");?>">
					
                </div>
			<?php } ?>
        </div>
        <div class="form-row">
			<?php if(!in_array("req_counter",$except)){ ?>
                <div class="form-group col-sm">
                    <label class="col-form-label" for="req_counter"><?php $this->_ee("Req Counter"); ?></label>
                    <input  class="form-control form-control-sm" type="text" maxlength="3"   value="<?php echo  $mainobj->GetPostValue("req_counter");?>" id="req_counter" <?php echo in_array("req_counter", $disabled)?' disabled="disabled" ':' name="req_counter" ';?>     placeholder="<?php $this->_ee("Req Counter");?>" data-bv-notempty="true" 	data-bv-notempty-message="<?php   $this->_ee("%s is required","Req Counter");?>">
					
                </div>
			<?php } ?>

			<?php if(!in_array("status",$except)){ ?>
                <div class="form-group col-sm">
                    <label class="col-form-label" for="status"><?php $this->_ee("Status"); ?></label>
                    <div class="form-row">
						<?php
						$status_selected= $mainobj->GetPostValue("status","B");
						$status_isDisabled=in_array("status", $disabled);
						APBD_GetHTMLRadioByArray("Status","status","status",true,$mainobj->GetPropertyOptions("status"),$status_selected,$status_isDisabled);
						?>
						
                    </div>
                </div>
			<?php } ?>

        </div>
		<?php
	}


}
?>