<?php 			
/** 
 * @since: 24/Feb/2019
 * @author: Sarwar Hasan 
 * @version 1.0.0
 * @property:id,name,email,country,company,entry_time,status		
 */						
class Mapbd_client extends AppsBDModel{
	public $id;
	public $name;
	public $username;
	public $email;
	public $country;
	public $company;
	public $entry_time;
	public $status;
    private $isDisabledEmailValidation=false;

	    /**
	     *@property id,name,email,country,company,entry_time,status
		 */
		function __construct() {
			parent::__construct ();
			$this->SetValidation();
			$this->tableName="apbd_client";
			$this->primaryKey="id";
			$this->uniqueKey=array(array("email"));
			$this->multiKey=array();
			$this->autoIncField=array("id");
			$this->app_base_name="APBLIC";
		}


	function SetValidation(){
		$this->validations=array(
			"id"=>array("Text"=>"ID", "Rule"=>"xss_clean|max_length[11]|integer"),
			"name"=>array("Text"=>"Name", "Rule"=>"xss_clean|required|max_length[100]"),
			"username"=>array("Text"=>"Username", "Rule"=>"xss_clean|max_length[150]"),
			"email"=>array("Text"=>"Email", "Rule"=>"required|xss_clean|max_length[150]|valid_email"),
			"country"=>array("Text"=>"Country", "Rule"=>"xss_clean|max_length[2]"),
			"company"=>array("Text"=>"Company", "Rule"=>"xss_clean|max_length[100]"),
			"entry_time"=>array("Text"=>"Entry Time", "Rule"=>"xss_clean|max_length[20]"),
			"status"=>array("Text"=>"Status", "Rule"=>"xss_clean|required|max_length[1]")
		);
	}
	function SetValidationForAPI(){
		    $this->SetValidation();
		    $this->validations['name']['Rule']="xss_clean|max_length[100]";
    }
	function DisableEmailInValidation() {
		    $this->isDisabledEmailValidation=true;
		if ( isset( $this->validations["email"] ) ) {
			unset( $this->validations["email"] );
		}
	}

	public function GetPropertyRawOptions($property,$isWithSelect=false){
	    $returnObj=array();
		switch ($property) {
	      case "status":
	         $returnObj=array("A"=>"Active","I"=>"Inactive");
	         break;
	      default:
	    }
        if($isWithSelect){
            return array_merge(array(""=>"Select"),$returnObj);
        }
        return $returnObj;

	}

 
	public function GetPropertyOptionsColor($property){
	    $returnObj=array();
		switch ($property) {
			case "status":
				$returnObj = array( "A" => "success", "I" => "danger" );
				break;
			default:
		}
        return $returnObj;

	}
 

	public function GetPropertyOptionsIcon($property){
	    $returnObj=array();
		switch ($property) {
			case "status":
				$returnObj = array( "A" => "fa fa-check-circle", "I" => "fa fa-times-circle" );
				break;
			default:
		}
        return $returnObj;

	}
 

 
	//auto generated
    function Save() {
		    if(!$this->isDisabledEmailValidation){
			    if ( $this->IsExists( "email", $this->email ) ) {
				    $this->AddError( "Email address already exists" );
				
				    return false;
			    }
            }
	   
	    if ( parent::Save() ) {
		    $keyword                  = "NCC";
		    $params                   = Mapbd_email_templates::getEmailParamListClearData( $keyword );
		    $params["client_name"]    = $this->name;///Client Name
		    $params["client_country"] = $this->country;//Country Name of Client
		    Mapbd_email_templates::SendEmailTemplates( $keyword, $this->email, $params );
		    
		    $client=Mapbd_client::FindBy("id",$this->id);
		    do_action(APBDLic::GetInstance()->getHookActionStr("new_client"),$client);
		    return true;
	    }
	
	    return false;
    }



	static function CreateDBTable(){
		$thisObj=new static();
		$table=$thisObj->db->prefix.$thisObj->tableName;
		if($thisObj->db->get_var("show tables like '{$table}'") != $table){
			$sql = "CREATE TABLE `{$table}` (
                  `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
                  `name` char(100) NOT NULL DEFAULT '',
                  `username` char(150) DEFAULT NULL,
                  `email` char(150) DEFAULT NULL,
                  `country` char(2) NOT NULL DEFAULT '' COMMENT 'ISO Alpha-2 Country Codes',
                  `company` char(100) NOT NULL DEFAULT '',
                  `entry_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
                  `status` char(1) NOT NULL COMMENT 'bool(A=Active, I=Inactive)',
                  PRIMARY KEY (`id`) USING BTREE,
                  UNIQUE KEY `email` (`email`) USING BTREE,
                  UNIQUE KEY `username` (`username`)
                ) DEFAULT CHARSET=utf8 COMMENT='APBDClient'";
			require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
			dbDelta($sql);
		}
	}
	function DropDBTable(){
		global $wpdb;
		$table=$wpdb->prefix.$this->tableName;
		$table_name = $wpdb->prefix . $this->tableName;
		$sql        = "DROP TABLE IF EXISTS $table_name;";
		$wpdb->query( $sql );
	}
	/* add custom function here*/
    static function DeleteByID( $id ) {
	   if(parent::DeleteByKeyValue( "id",$id)){
	       Mapbd_license::DeleteByClient($id);
	       return true;
       }else{
		   return false;
       }
    }
    static function UpdateClientEmail($id,$email){
        $checkobj=new self();
        if(!$checkobj->IsExists("email",$email)) {
	        $Obj = new self();
	        $Obj->email( $email );
	        $Obj->SetWhereUpdate( "id", $id );
	
	        return $Obj->Update();
        }else{
            return false;
        }
    }
	
	/* end custom function */
	function GetAddForm($label_col=5,$input_col=7,$mainobj=null,$except=array(),$disabled=array()){

		if(!$mainobj){
			$mainobj=$this;
		}
		?>
        <div class="form-row">
	        <?php if(!in_array("name",$except)){ ?>
                <div class="form-group col-sm">
                    <label class="col-form-label" for="name"><?php $this->_ee("Name"); ?></label>
                    <input  class="form-control form-control-sm" type="text" maxlength="100"   value="<?php echo  $mainobj->GetPostValue("name");?>" id="name" <?php echo in_array("name", $disabled)?' disabled="disabled" ':' name="name" ';?>     placeholder="<?php $this->_ee("Name");?>" data-bv-notempty="true" 	data-bv-notempty-message="<?php   $this->_ee("%s is required","Name");?>">
			        
                </div>
	        <?php } ?>
			<?php if(!in_array("company",$except)){ ?>
                <div class="form-group col-sm">
                    <label class="col-form-label" for="company"><?php $this->_ee("Company"); ?></label>
                    <input  class="form-control form-control-sm" type="text" maxlength="100"   value="<?php echo  $mainobj->GetPostValue("company");?>" id="company" <?php echo in_array("company", $disabled)?' disabled="disabled" ':' name="company" ';?>     placeholder="<?php $this->_ee("Company");?>">
					
                </div>
			<?php } ?>
        </div>
        <div class="form-row">
			<?php if(!in_array("email",$except)){ ?>
                <div class="form-group col-sm">
                    <label class="col-form-label" for="email"><?php $this->_ee("Email"); ?></label>
                    <input  class="form-control form-control-sm" type="text" maxlength="150"   value="<?php echo  $mainobj->GetPostValue("email");?>" id="email" <?php echo in_array("email", $disabled)?' disabled="disabled" ':' name="email" ';?>     placeholder="<?php $this->_ee("Email");?>" data-bv-notempty="true" 	data-bv-notempty-message="<?php   $this->_ee("%s is required","Email");?>">
					
                </div>
			<?php } ?>
         
	        <?php if(!in_array("country",$except)){ ?>
                <div class="form-group col-sm">
                    <label class="col-form-label" for="country"><?php $this->_ee("Country"); ?></label>
                 
                    <select data-live-search="true"  class="app-select-picker form-control-sm " id="country" <?php echo in_array("country", $disabled)?' disabled="disabled" ':' name="country" ';?> >
                        <option value=""><?php $this->_ee("Select"); ?></option>
		                <?php $country_selected= $mainobj->GetPostValue("country");
			                APBD_GetHTMLOptionByArray(APBD_getCountryKeyValuePair(),$country_selected);
		                ?>
                    </select>
                    
                   
                </div>
	        <?php } ?>

        </div>
        <div class="form-row">

			<?php if(!in_array("status",$except)){ ?>
                <div class="form-group col-sm">
                    <label class="col-form-label" for="status"><?php $this->_ee("Status"); ?></label>

					<?php
					APBD_GetHTMLSwitchButton("status","status","I","A",$mainobj->GetPostValue("status"));
					?>
					<?php /*<span class="form-text text-muted"><?php $this->_ee("status");?></span>	*/?>


                </div>
			<?php } ?>

        </div>
		<?php
	}



}
?>