<?php
//Elite Licenser
class APBDLicenseSettings  extends  AppsBDBaseModule {
	public $isAPIResponsePage = false;
	
	public function initialize() {
		parent::initialize();
		$this->disableDefaultForm();
		$this->AddAjaxAction( "create_page", [ $this, "create_page" ] );
		$this->AddAjaxAction( "force_check_update", [ $this, "force_check_update" ] );
		$this->AddAjaxAction( "get-doc", [ $this, "get_documentation" ] );
		$this->AddAjaxAction( "refresh-app", [ $this, "refresh_app" ] );
		$this->AddAjaxAction( "force-db-upgrade", [ $this, "forceDBModification" ] );
		$this->AddAjaxNoPrivAction( "force-db-upgrade", [ $this, "forceDBModification" ] );
		$this->AddAjaxAction( "send-test-hook", [ $this, "TestHook" ] );
		$this->AddAjaxAction( "update-api-msg", [ $this, "update_api_msg" ] );
		
		$this->setIsLastMenu( true );
		
		//$this->addTopMenu("Add-Ons","fa fa-puzzle-piece",'#APBDELAddons',"btn btn-info text-white",true);
		$this->addTopMenu(
			"Refresh App", "fa fa-gear",
			$this->GetActionUrl("refresh-app"),
			"btn btn-info ConfirmAjaxWR text-white", false,['data-msg'=>"Are you sure to refresh app?"]);
		$this->addTopMenu( "Documentation", "ap ap-documentation", [
			$this,
			"documentation"
		], "btn btn-success apbd-lazy-loader", true, [ "id" => "applicdoc", "data-onclick" => "ApplicLoadDocument" ] );
		
		
		add_action( 'wp_dashboard_setup', function () {
			wp_add_dashboard_widget( 'elite_licenser', '<i class="abp abp-elite-licenser"></i>' . $this->__( "Elite Licenser" ), [
				$this,
				'license_summery'
			] );
		} );
		add_action( $this->getHookActionStr( 'new_license' ), [ $this, "hook_new_license" ] );
		add_action( $this->getHookActionStr( 'new_client' ), [ $this, "hook_new_client" ] );
		add_action("elite-licenser/license/add",[$this,"add_license_data"]);
		add_filter("elite-licenser/api-key/verify",[$this,"is_valid_api_key"],1,3);
		add_action("apbd-db-query-error",function($model,$error_message){
			self::DoVersionUpdate( "", "", true );
        },10,2);
		add_action( 'upgrader_process_complete', function($obj, $hook_extra){
		    if($hook_extra['action']=='update' && $hook_extra['action']=='plugin' ){
		        foreach ($hook_extra['plugins'] as $plugin_slug){
		            if($plugin_slug=='elite-licenser/elitelicenser.php' || $plugin_slug=='elitelicenserlite/elitelicenserlite.php' ){
			            self::DoVersionUpdate( "", "", true );
			            break;
                    }
                }
            }
        }, 10, 2 );
		
		
		$this->setExtraMarkets();
		
	}
	public function OnInit() {
	    parent::OnInit();
		$api = new APBDLicAPI( $this );
		$api->isOk();
		//add_filter( 'rest_url_prefix', [$this,'my_theme_api_slug']);
		add_action( 'rest_api_init', [ $api, 'routes' ] );
		add_filter('elite-licenser/filter/check-active-request',[$this,"check_local_domain"],10,2);
		add_filter('elite-licenser/filter/domain-log',[$this,"domain_log_filter"]);
		add_filter($this->getHookActionStr("before-active-response"),[$this,'check_unlimited'],99,1);
	}
	public function check_unlimited($licenseObj){
		if(!empty($licenseObj->is_unlimited) && $licenseObj->is_unlimited=='Y'){
			$licenseObj->max_domain=999999;
		}
		return $licenseObj;
    }
	
	public function check_local_domain($ob,$postData){
		if($this->GetOption( "allow_test_domain", "N" )=='Y'){
			$domain = parse_url($postData->domain);
			if(!empty($domain['host']) && el_is_test_domain($domain['host'])) {
				if ( empty( $postData->product_id ) ) {
					$ob->license = Mapbd_license::FindBy( "product_base_name", $postData->product_base, [ "purchase_key" => $postData->license_key ] );
				} else {
					$ob->license = Mapbd_license::FindBy( "product_id", $postData->product_id, [ "purchase_key" => $postData->license_key ] );
				}
				if ( ! empty( $ob->license ) ) {
					$ip = APBD_get_remote_ip();
					Mapbd_domain_log::Add( $ob->license->purchase_key, $postData->domain, $ip, $postData->app_version, 'A', $postData->email );
					$ob->status = true;
				}
			}
        }
		return $ob;
    }
	public function domain_log_filter($domain) {
		if ( $this->GetOption( "allow_test_domain", "N" ) == 'Y' ) {
			$domain_info = parse_url($domain);
			if(!empty($domain_info['host']) && el_is_test_domain($domain_info['host'])) {
				$domain='[Test Domain] ' .$domain;
			}
		}
		return $domain;
	}
	function refresh_app() {
		self::DoVersionUpdate( "", "", true );
		$mainResponse = new AppsbdAjaxConfirmResponse();
		$mainResponse->DisplayWithResponse(true,"Successfully refreshed");
	}
	
	function forceDBModification() {
		self::DoVersionUpdate( "", "", true );
		echo "DB MODIFICATION DONE";
		die;
	}
	
	function license_summery() {
		$licState = Mapbd_license::getLicenseStats();
		?>
        <div class="apbd-el-mini-rp">
            <table class="apbd-rp-table">
                <tr>
                    <th><?php $this->_e( "Total Active License" ); ?></th>
                    <th class="apbd-th-div">:</th>
                    <td><?php echo $licState->active; ?></td>
                </tr>
                <tr>
                    <th><?php $this->_e( "Total Free License" ); ?></th>
                    <th class="apbd-th-div">:</th>
                    <td><?php echo $licState->free; ?></td>
                </tr>
                <tr>
                    <th><?php $this->_e( "Total Inactive License" ); ?></th>
                    <th class="apbd-th-div">:</th>
                    <td><?php echo $licState->inactive; ?></td>
                </tr>
                <tr>
                    <th> <?php $this->_e( "Last License Add Time" ); ?> </th>
                    <th class="apbd-th-div">:</th>
                    <td><?php echo APBD_getWPDateTimeWithFormat( $licState->last_entry_time ); ?></td>
                </tr>
				<?php if ( ! empty( $licState->products ) ) { ?>
                    <tr>
                        <th class="table-title"
                            colspan="3"> <?php $this->_e( "To %d Products", count( $licState->products ) ); ?> </th>
                    </tr>
					<?php foreach ( $licState->products as $product ) {
						?>
                        <tr>
                            <th> <?php echo $product->name; ?> </th>
                            <th class="apbd-th-div">:</th>
                            <td><?php echo $product->total; ?></td>
                        </tr>
						<?php
					}
				} ?>
            </table>
            <br>
            <br>

        </div>
		<?php
	}
	
	function setExtraMarkets() {
		$extraMarkets = $this->GetOption( "extra_mkt_dtls", "" );
		if ( ! empty( $extraMarkets ) ) {
			$extraMarkets = explode( ",", $extraMarkets );
			if ( is_array( $extraMarkets ) && count( $extraMarkets ) > 0 ) {
				foreach ( $extraMarkets as $extra_market ) {
					$mkts = explode( "=", $extra_market );
					if ( is_array( $mkts ) && count( $mkts ) == 2 ) {
						Mapbd_license::RegisterMarkets( trim( $mkts[0] ), trim( $mkts[1] ) );
					}
				}
			}
		}
	}
	
	private function setHookSignature( &$params ,$private_key='') {
	    if(isset($params['_signature_'])){
		    unset( $params['_signature_'] );
        }
		$strCode      = serialize($params) . $private_key;
		$params['_signature_'] =md5($strCode);
	}
	
	public function hook_new_license( $hookObj ) {
		$licenseObj = ! empty( $hookObj->license ) ? $hookObj->license : new Mapbd_license();
		$client     = ! empty( $hookObj->client ) ? $hookObj->client : new Mapbd_client();
		$product    = ! empty( $hookObj->product ) ? $hookObj->product : new Mapbd_product();
		if ( empty( $licenseObj ) ) {
			$licenseObj = new Mapbd_license();
		} elseif ( ! ( $licenseObj instanceof Mapbd_license ) ) {
			$licenseObj = APBD_CastClass( "Mapbd_license", $licenseObj );
		}
		if ( empty( $client ) ) {
			$client = new Mapbd_client();
		} elseif ( ! ( $client instanceof Mapbd_client ) ) {
			$client = APBD_CastClass( "Mapbd_client", $client );
		}
		
		$params   = [];
		$licArray = $licenseObj->getPropertiesArray();
		foreach ( $licArray as $lic_key => $licValue ) {
			if(is_null($licValue)){
				$licValue='';
			}
			$params[ "lic_" . $lic_key ] = $licValue;
		}
		
		$params["lic_status_description"] =  $licenseObj->GetPropertyRawOptions( "status" );
		$params["lic_market_description"] =  $licenseObj->GetPropertyRawOptions( "market" );
		
		$clientArray = $client->getPropertiesArray();
		foreach ( $clientArray as $cl_key => $clValue ) {
			if(is_null($clValue)){
				$clValue='';
			}
			$params[ "client_" . $cl_key ] = $clValue;
		}
		if ( ! isset( $params['client_email'] ) ) {
			$params['client_email'] = "";
		}
		if ( ! isset( $params['lic_product_name'] ) ) {
		    if(empty($product->product_name)){
			    $product=Mapbd_product::FindBy("id",$product->id);
            }
			$params['lic_product_name'] = !empty($product->product_name)?$product->product_name:"";
		}
		$isSkipEmail=APBD_PostValue("skip_nl_email","");
		if(empty($isSkipEmail)) {
			Mapbd_email_templates::SendEmailTemplates( "NLC", $client->email, $params );
		}
		
		$mhooks=Mapbd_webhook::FindAllBy("status","A",["is_on_license"=>"A"]);
		foreach ( $mhooks as $mhook ) {
			$this->setHookSignature( $params ,$mhook->private_key);
			$response = wp_remote_post( $mhook->url, array(
					'method'  => 'POST',
					'timeout' => 120,
					'body'    => $params
				)
			);
			
			if ( is_wp_error( $response ) ) {
				//add log
			} else {
				//add log
			}
		}
	}
	
	public function hook_new_client( $client ) {
		if ( empty( $client ) ) {
			$client = new Mapbd_client();
		} elseif ( ! ( $client instanceof Mapbd_client ) ) {
			$client = APBD_CastClass( "Mapbd_client", $client );
		}
		$data = $client->getPropertiesArray();
		if ( ! isset( $data['email'] ) ) {
			$data['email'] = "";
		}
		$params = [];
		foreach ( $data as $key => $val ) {
		    if(is_null($val)){
			    $val='';
		    }
			$params[ 'client_' . $key ] = $val;
		}
		$params  = apply_filters( $this->getHookActionStr( "web-hook-client" ), $params );
		if(!is_array($params)){
		    return;
		}
		$webhooks = Mapbd_webhook::FindAllBy( "status", "A", [ "is_on_client" => "A" ] );
		foreach ( $webhooks as $webhook ) {
			$this->setHookSignature( $params, $webhook->private_key );
			$response = wp_remote_post( $webhook->url, array(
					'method'  => 'POST',
					'timeout' => 120,
					'body'    => $params
				)
			);
			if ( is_wp_error( $response ) ) {
				//add log
			} else {
				//add log
			}
		}
	}
	
	public function OnTableCreate() {
		parent::OnTableCreate();
		Mapbd_apikey::CreateDBTable();
		Mapbd_ban_host::CreateDBTable();
		Mapbd_client::CreateDBTable();
		Mapbd_domain_log::CreateDBTable();
		Mapbd_license::CreateDBTable();
		Mapbd_license_type::CreateDBTable();
		Mapbd_product::CreateDBTable();
		Mapbd_product_license::CreateDBTable();
		Mapbd_product_update::CreateDBTable();
		Mapbd_request_manage::CreateDBTable();
		Mapbd_updateinfo::CreateDBTable();
		Mapbd_useddomain::CreateDBTable();
		Mapbd_upload::CreateDBTable();
		Mapbd_webhook::CreateDBTable();
		Mapbd_paddle_log::CreateDBTable();
		
	}
	
	public static function DoVersionUpdate( $current_version, $previous_version, $isForce = false ) {
		if ( $isForce || version_compare( $previous_version, "1.0.10", '<' ) ) {
			Mapbd_license::DBColumnAddOrModify( "extra_param", "char", "100", "''", "NOT NULL", "market", "", "" );
			Mapbd_domain_log::DBColumnAddOrModify( "client_email", "char", "100", "''", "NOT NULL", "entry_time", "", "" );
		}
		if ( $isForce || version_compare( $previous_version, "2.0.0", '<' ) ) {
			Mapbd_upload::CreateDBTable();
		}
		if ( $isForce || version_compare( $previous_version, "2.0.2", '<' ) ) {
			Mapbd_product::DBColumnAddOrModify( "renew_link", "varchar", "255", "''", "NOT NULL", "enc_key", "", "" );
			Mapbd_product::DBColumnAddOrModify( "is_enc_up_res", "char", "1", "'N'", "NOT NULL", "renew_link", "bool(Y=Yes, N=No)", "" );
			Mapbd_product::DBColumnAddOrModify( "is_check_lic_on_update", "char", "1", "'N'", "NOT NULL", "renew_link", "bool(Y=Yes, N=No)", "" );
			Mapbd_apikey::DBColumnAddOrModify( "title", "varchar", "255", "''", "NOT NULL", "api_ip", "", "" );
			Mapbd_license::DBColumnAddOrModify( "lictype_param", "char", "20", "''", "NOT NULL", "extra_param", "", "" );
			Mapbd_license::DBColumnAddOrModify( "other_param", "char", "20", "''", "NOT NULL", "lictype_param", "", "" );
			Mapbd_license_type::DBColumnAddOrModify( "extra_param", "char", "20", "''", "NOT NULL", "days_of_support", "", "" );
			if ( self::GetModuleOption( "is_chk_lic_on_up" ) == "Y" ) {
				$product = new Mapbd_product();
				$product->is_check_lic_on_update( 'Y' );
				$product->SetWhereUpdate( "status", 'A' );
				if ( $product->Update( true ) ) {
				
				}
			}
		}
		//no need is force
		if ($isForce || version_compare( $previous_version, "2.0.4", '<' ) ) {
			set_transient( "_el_fs_link_cng", 1);
			Mapbd_webhook::CreateDBTable();
			Mapbd_product::DBColumnAddOrModify( "product_link", "varchar", "255", "''", "NOT NULL", "renew_link", "", "" );
		}
		if ($isForce || version_compare( $previous_version, "2.2.6", '<' ) ) {
			Mapbd_license::DBColumnAddOrModify( "extra_param2", "char", "150", "''", "NOT NULL", "extra_param", "", "" );
			Mapbd_paddle_log::CreateDBTable();
		}
		if ($isForce || version_compare( $previous_version, "2.2.8", '<' ) ) {
			Mapbd_license::DBColumnAddOrModify( "active_time", "timestamp", "", "CURRENT_TIMESTAMP", "NOT NULL", "entry_time", "", "" );
		}
		if ($isForce || version_compare( $previous_version, "2.3.0", '<' ) ) {
			Mapbd_product::DBColumnAddOrModify( "is_check_sup_on_update", "char", "1", "'N'", "NOT NULL", "is_check_lic_on_update", "bool(Y=Yes, N=No)", "" );
		}
		
		if ($isForce || version_compare( $previous_version, "2.4.0", '<' ) ) {
			Mapbd_product::DBColumnAddOrModify( "lic_prefix", "varchar", "10", "''", "NOT NULL", "renew_link", "", "" );
			Mapbd_license_type::DBColumnAddOrModify( "is_unlimited", "char", "1", "'N'", "NOT NULL", "max_domain", "", "" );
			Mapbd_license::DBColumnAddOrModify( "is_unlimited", "char", "1", "'N'", "NOT NULL", "max_domain", "", "" );
		}
	}
	
	function OnPluginVersionUpdated( $current_version, $previous_version ) {
		self::DoVersionUpdate( $current_version, $previous_version, false );
	}
	
	function my_theme_api_slug( $slug ) {
		flush_rewrite_rules( true );
		return $slug;
	}
	
	function OptionForm() {
		$this->Display();
	}
	
	public function documentation() {
		$this->Display( "documentation" );
	}
	
	public function get_documentation() {
		$this->Display( "get-doc" );
		exit;
	}
	
	function TestHook() {
		$param        = APBD_PostValue( "param", "" );
		$type         = APBD_PostValue( "type", "" );
		$mainResponse = new AppsbdAjaxConfirmResponse();
		if ( ! empty( $param ) && ! empty( $type ) ) {
			if ( $type == "lic" ) {
				$licenseObject = Mapbd_license::FindBy( "id", $param );
				if ( empty( $licenseObject ) ) {
					$mainResponse->DisplayWithResponse( false, $this->__( "Invalid License Information" ), NULL, " ap ap-hook", $this->__( "Sent Test Request" ) );
				}
				$client = Mapbd_client::FindBy( "id", $licenseObject->client_id );
				if ( empty( $client ) ) {
					$client = new Mapbd_client();
				}
				$lichook          = new stdClass();
				$lichook->license = $licenseObject;
				$lichook->client  = $client;
				do_action( $this->getHookActionStr( "new_license" ), $lichook );
				$mainResponse->DisplayWithResponse( true, $this->__( "Sent Test License Request" ), NULL, " ap ap-hook", $this->__( "Sent Test Request" ) );
				
			} elseif ( $type == "cl" ) {
				$client = Mapbd_client::FindBy( "id", $param );
				do_action( $this->getHookActionStr( "new_client" ), $client );
				$mainResponse->DisplayWithResponse( true, $this->__( "Sent Test Client Request" ), NULL, " ap ap-hook", $this->__( "Sent Test Request" ) );
			} else {
				$mainResponse->DisplayWithResponse( false, $this->__( "Unknown type request" ), NULL, " ap ap-hook", $this->__( "Sent Test Request" ) );
			}
		} else {
			$mainResponse->DisplayWithResponse( false, $this->__( "Invalid request param" ) );
		}
	}
	function update_api_msg() {
		$key        = APBD_PostValue( "name", "" );
		$val         = APBD_PostValue( "val", "" );
		$mainResponse = new AppsbdAjaxConfirmResponse();
		if(!empty($key) && !empty($val) ) {
			$old_msg      = self::GetAPIMessages( $key );
			if($old_msg ==$val){
				$mainResponse->DisplayWithResponse( false, $this->__( "No change for update" ) );
			}else{
			    $oldParamCount=substr_count($old_msg,'%s');
			    if($oldParamCount==substr_count($val,'%s')){
			        //update
                    $api_message=get_option('elite_api_msg',[]);
                    $api_message[$key]=$val;
                    if(update_option('elite_api_msg',$api_message) || add_option('elite_api_msg',$api_message)) {
	                    $mainResponse->DisplayWithResponse( true, 'Successfully updated' );
                    }
			    }else{
				    $mainResponse->DisplayWithResponse( false, $this->__( '(%s) count does not match with default message, it has %s %s in default message' ,'%s',$oldParamCount,'%s') );
			    }
			}
			
		}else{
			$mainResponse->DisplayWithResponse( false, $this->__( "Invalid request param" ) );
		}
	}
	public function PluginRowMeta( &$links ) {
		//$links[] = " <a class='edit coption' href='".esc_url($this->GetActionUrl("force_check_update"))."'>".$this->__("Update Check")."</a><br/><br/>";
	}
	
	function force_check_update() {
		update_option( '_site_transient_update_plugins', '' );
		wp_redirect( admin_url( 'plugins.php' ) );
		exit;
	}
	
	function create_page() {
		$response = new AppsbdAjaxConfirmResponse();
		$new_post = array(
			'post_title'    => $this->__( "API CALL" ),
			'post_content'  => "Create for api",
			'post_status'   => 'publish',
			'post_date'     => date( 'Y-m-d H:i:s' ),
			'post_author'   => get_current_user_id(),
			'post_type'     => 'page',
			'post_category' => ''
		);
		$post_id  = wp_insert_post( $new_post );
		$this->AddIntoOption( "apicallpageid", $post_id );
		$this->UpdateOption();
		$res             = new stdClass();
		$res->page_id    = $post_id;
		$res->page_title = $new_post['post_title'];
		$response->DisplayWithResponse( true, "this is a test", $res );
	}
	
	public function OnActive() {
		parent::OnActive();
		$encKey = $this->GetOption( "api_enc_key", "" );
		if ( empty( $encKey ) ) {
			$this->AddOption( "api_enc_key", strtoupper( hash( 'crc32b', rand( 1000, 9999 ) ) ) );
		}
		flush_rewrite_rules(true);
		
	}
	function is_valid_api_key($is_ok,$apiKey,$type){
		$type = strtolower( $type );
		if ( ! empty( $apiKey ) ) {
			$postbackup=$_POST;
			$apiObj = Mapbd_apikey::FindBy( "api_key", $apiKey, [ "status" => 'A' ] );
			$_POST=$postbackup;
			if ( $apiObj ) {
				$ip = APBD_get_remote_ip();
				if ( ! empty( $apiObj->api_ip ) && $ip != $apiObj->api_ip ) {
					$response = new AppsbdAPIResponse();
					$response->DisplayWithResponse( false, $this->__( "Invalid request IP" ) );
				}
				if ( ! empty( $type ) ) {
					if ( ! empty( $apiObj->{$type} ) && $apiObj->{$type} == "Y" ) {
						return true;
					}
					
				}
			}
		}
		$response = new AppsbdAPIResponse();
		$response->DisplayWithResponse( false, $this->__( "The API key doesn't have this operation's permission" ) );
	}
	function add_license_data( $licenseData = [] ) {
		$return_license_only = $licenseData['return_license_only'];
		$seperator           = "";
		if(!empty($licenseData['seperator'])){
			$seperator=$licenseData['seperator'];
		}elseif(!empty($licenseData['separator'])){
			$seperator=$licenseData['separator'];
		}
		if ( empty( $seperator ) || $seperator == "-" ) {
			$seperator = "\n";
		}
		$response=$this->add_license_data_return($licenseData);
		if($response->status &&  !empty( $return_license_only ) ) {
			if ( empty( $seperator ) || $seperator == "-" ) {
				$seperator = "\n";
			}
			echo implode( $seperator, $response->data->license_code );
		}else{
		    $response->Display();
		}
		
	}
	
	/**
	 * @param array $licenseData
	 *
	 * @return AppsbdAPIResponse
	 */
	function add_license_data_return( $licenseData = [] ) {
		//$this->APPSBDLoadDatabaseModel( "Mapbd_client" );
		
		$isOk                = true;
		$response            = new AppsbdAPIResponse();
		$client_name         = $licenseData['client_name'];
		$client_email        = $licenseData['client_email'];
		$client_country      = $licenseData['client_country'];
		$client_company      = $licenseData['client_company'];
		$product_id          = $licenseData['product_id'];
		$product_base        = $licenseData['product_base'];
		$license_type_id     = $licenseData['license_type_id'];
		$market              = $licenseData['market'];
		$extra_param         = ! empty( $licenseData['extra_param'] ) ? $licenseData['extra_param'] : "";
		$extra_param2        = ! empty( $licenseData['extra_param2'] ) ? $licenseData['extra_param2'] : "";
		$qty                 = $licenseData['qty'];
		$required = "";
		if ( empty( $client_email ) ) {
			$isOk     = false;
			$required .= "Client email";
		}
		if ( empty( $product_id ) && empty( $product_base ) ) {
			$isOk     = false;
			$required .= ! empty( $required ) ? ", " : "";
			$required .= "Product id or base";
		}
		if ( empty( $license_type_id ) ) {
			$isOk     = false;
			$required .= ! empty( $required ) ? " and " : "";
			$required .= "License type id";
		}
		if ( ! $isOk ) {
			$response->SetResponse( false, sprintf( "%s is required", $required ) );
			
			return $response;
		}
		$client    = Mapbd_client::FindBy( "email", $client_email );
		$client_id = "";
		if ( $client ) {
			$client_id = $client->id;
		} else {
			//new
			$nc = new Mapbd_client();
			$nc->SetValidationForAPI();
			$nc->email( $client_email );
			$nc->name( $client_name );
			$nc->status( 'A' );
			$nc->country( $client_country );
			$nc->company( $client_company );
			if ( $nc->Save() ) {
				$client_id = $nc->id;
			} else {
				$response->SetResponse( false, APBD_GetMsg_API() );
				
				return $response;
			}
		}
		if ( ! empty( $product_id ) ) {
			$product = Mapbd_product::FindBy( "id", $product_id );
		} else {
			$product = Mapbd_product::FindBy( "product_base", $product_base );
		}
		if ( empty( $product ) || $product->status != 'A' ) {
			$response->SetResponse( false, "Product information doesn't exist" );
			
			return $response;
		}
		$product_license = Mapbd_product_license::FindBy( "product_id", $product_id, [ 'license_id' => $license_type_id ] );
		if ( empty( $product_license ) || $product_license->status != 'A' ) {
			$response->SetResponse( false, "license type doesn't associated this product" );
			
		}
		
		$licenseObj = Mapbd_license_type::FindBy( "id", $license_type_id );
		if ( empty( $licenseObj ) || $licenseObj->status != 'A' ) {
			$response->SetResponse( false, "license type doesn't exists" );
			
			return $response;
		}
		$hasError     = false;
		$licenseCodes = [];
		for ( $iq = 0; $iq < $qty; $iq ++ ) {
			//now we are inserting license
			$liceobj = new Mapbd_license();
			$liceobj->client_id( $client_id );
			$liceobj->product_id( $product->id );
			$liceobj->product_base_name( $product->product_base );
			$liceobj->license_id( $licenseObj->id );
			$liceobj->market( $market );
			$liceobj->license_title( $licenseObj->license_title );
			$liceobj->max_domain( $licenseObj->max_domain );
			$liceobj->verification_required( $licenseObj->verification_required );
			$liceobj->lictype_param( $licenseObj->extra_param );
			$liceobj->is_unlimited($licenseObj->is_unlimited);
			
			if ( ! empty( $licenseData['expiry_time'] ) ) {
				$liceobj->has_expiry( 'Y' );
				$liceobj->expiry_time( $licenseData['expiry_time'] );
			} else {
				$liceobj->has_expiry( $licenseObj->has_expiry );
				if ( $licenseObj->has_expiry == "Y" ) {
					$liceobj->expiry_time( date( 'Y-m-d H:i:s', strtotime( "+" . $licenseObj->days_of_expiry . " DAYS" ) ) );
				}
			}
			$liceobj->has_support( $licenseObj->has_support );
			if ( $licenseObj->has_support == "Y" ) {
				if ( ! empty( $licenseData['expiry_time'] ) ) {
					$liceobj->support_end_time( date( 'Y-m-d H:i:s', strtotime( $licenseData['expiry_time']) ));
				}else{
				    $liceobj->support_end_time( date( 'Y-m-d H:i:s', strtotime( "+" . $licenseObj->days_of_support . " DAYS" ) ) );
				}
			}
			//$liceobj->entry_time();
			//$request_obj=new Mapbd_request_manage();
			
			//$liceobj->request_delay_time();
			$liceobj->status( 'W' );
			if ( ! empty( $extra_param ) ) {
				$liceobj->extra_param( $extra_param );
			}
			$liceobj->extra_param2( $extra_param2 );
			if ( $liceobj->Save() ) {
				$licenseCodes[] = $liceobj->purchase_key;
				
			} else {
				$hasError = true;
				break;
				
			}
		}
		if ( ! $hasError ) {
			$responseData                = new stdClass();
			$responseData->product_name  = $product->product_name;
			$responseData->license_code  = $licenseCodes;
			$responseData->license_title = $licenseObj->license_title;
			$responseData->client_id     = $client_id;
			$response->SetResponse( true, "License successfully added", $responseData );
			return $response;
		} else {
			$response->SetResponse( false, APBD_GetMsg_API() );
			
			return $response;
		}
	}
	function add_license_data_bk( $licenseData = [] ) {
		//$this->APPSBDLoadDatabaseModel( "Mapbd_client" );
		$isOk                = true;
		$response            = new AppsbdAPIResponse();
		$client_name         = $licenseData['client_name'];
		$client_email        = $licenseData['client_email'];
		$client_country      = $licenseData['client_country'];
		$client_company      = $licenseData['client_company'];
		$product_id          = $licenseData['product_id'];
		$product_base        = $licenseData['product_base'];
		$license_type_id     = $licenseData['license_type_id'];
		$market              = $licenseData['market'];
		$extra_param         = !empty($licenseData['extra_param'])?$licenseData['extra_param']:"";
		$extra_param2         = !empty($licenseData['extra_param2'])?$licenseData['extra_param2']:"";
		$qty                 = $licenseData['qty'];
		$return_license_only = $licenseData['return_license_only'];
		$seperator           = $licenseData['seperator'];
		
		if ( empty( $seperator ) || $seperator == "-" ) {
			$seperator = "\n";
		}
		$required = "";
		if ( empty( $client_email ) ) {
			$isOk     = false;
			$required .= "Client email";
		}
		if ( empty( $product_id ) && empty( $product_base ) ) {
			$isOk     = false;
			$required .= ! empty( $required ) ? ", " : "";
			$required .= "Product id or base";
		}
		if ( empty( $license_type_id ) ) {
			$isOk     = false;
			$required .= ! empty( $required ) ? " and " : "";
			$required .= "License type id";
		}
		if ( ! $isOk ) {
			$response->DisplayWithResponse( false, sprintf( "%s is required", $required ) );
		}
		$client    = Mapbd_client::FindBy( "email", $client_email );
		$client_id = "";
		if ( $client ) {
			$client_id = $client->id;
		} else {
			//new
			$nc = new Mapbd_client();
			$nc->SetValidationForAPI();
			$nc->email( $client_email );
			$nc->name( $client_name );
			$nc->status( 'A' );
			$nc->country( $client_country );
			$nc->company( $client_company );
			if ( $nc->Save() ) {
				$client_id = $nc->id;
			} else {
				$response->DisplayWithResponse( false, APBD_GetMsg_API() );
			}
		}
		if ( ! empty( $product_id ) ) {
			$product = Mapbd_product::FindBy( "id", $product_id );
		} else {
			$product = Mapbd_product::FindBy( "product_base", $product_base );
		}
		if ( empty( $product ) || $product->status != 'A' ) {
			$response->DisplayWithResponse( false, "Product information doesn't exist" );
		}
		$product_license = Mapbd_product_license::FindBy( "product_id", $product_id, [ 'license_id' => $license_type_id ] );
		if ( empty( $product_license ) || $product_license->status != 'A' ) {
			$response->DisplayWithResponse( false, "license type doesn't associated this product" );
		}
		
		$licenseObj = Mapbd_license_type::FindBy( "id", $license_type_id );
		if ( empty( $licenseObj ) || $licenseObj->status != 'A' ) {
			$response->DisplayWithResponse( false, "license type doesn't exists" );
		}
		$hasError     = false;
		$licenseCodes = [];
		for ( $iq = 0; $iq < $qty; $iq ++ ) {
			//now we are inserting license
			$liceobj = new Mapbd_license();
			$liceobj->client_id( $client_id );
			$liceobj->product_id( $product->id );
			$liceobj->product_base_name( $product->product_base );
			$liceobj->license_id( $licenseObj->id );
			$liceobj->market( $market );
			//$liceobj->purchase_key();
			//$liceobj->user_domain("");
			//$liceobj->server_ip();
			//$liceobj->current_version();
			$liceobj->license_title( $licenseObj->license_title );
			$liceobj->max_domain( $licenseObj->max_domain );
			$liceobj->verification_required( $licenseObj->verification_required );
			$liceobj->lictype_param( $licenseObj->extra_param );
			
			if ( ! empty( $licenseData['expiry_time'] ) ) {
				$liceobj->has_expiry( 'Y' );
				$liceobj->expiry_time( $licenseData['expiry_time'] );
			}else {
				$liceobj->has_expiry( $licenseObj->has_expiry );
				if ( $licenseObj->has_expiry == "Y" ) {
					$liceobj->expiry_time( date( 'Y-m-d H:i:s', strtotime( "+" . $licenseObj->days_of_expiry . " DAYS" ) ) );
				}
			}
			$liceobj->has_support( $licenseObj->has_support );
			if ( $licenseObj->has_support == "Y" ) {
				$liceobj->support_end_time( date( 'Y-m-d H:i:s', strtotime( "+" . $licenseObj->days_of_support . " DAYS" ) ) );
			}
			//$liceobj->entry_time();
			//$request_obj=new Mapbd_request_manage();
			
			//$liceobj->request_delay_time();
			$liceobj->status( 'W' );
			if ( ! empty( $extra_param ) ) {
				$liceobj->extra_param( $extra_param );
			}
			$liceobj->extra_param2( $extra_param2 );
			if ( $liceobj->Save() ) {
				$licenseCodes[] = $liceobj->purchase_key;
				
			} else {
				$hasError = true;
				break;
				
			}
		}
		if ( ! $hasError ) {
			if ( empty( $return_license_only ) ) {
				$responseData                = new stdClass();
				$responseData->product_name  = $product->product_name;
				$responseData->license_code  = $licenseCodes;
				$responseData->license_title = $licenseObj->license_title;
				$responseData->client_id     = $client_id;
				$response->DisplayWithResponse( true, "License successfully added", $responseData );
			} else {
				echo implode( $seperator, $licenseCodes );
				die;
			}
		} else {
			$response->DisplayWithResponse( false, APBD_GetMsg_API() );
		}
	}
	function GetMenuTitle() {
		
		return $this->__( 'Settings' );
	}
	
	function GetMenuSubTitle() {
		return $this->__( 'Plugins Basic Settings' );
	}
	
	function GetMenuIcon() {
		return 'ap ap-setting';
	}
	
	function OnAdminMainOptionScripts() {
		//$this->AddAdminScript("test.js","testid");
	}
	
	public function AjaxRequestCallback() {
		if ( $this->kernelObject->isDemoMode() ) {
			$this->AddError( "This feature has been disabled in demo mode" );
			$response = new AppsbdAjaxConfirmResponse();
			$response->DisplayWithResponse( false, "Update has been disabled in demo mode" );
		} else {
			parent::AjaxRequestCallback();
		}
	}
	public function UpdateOption() {
		if(parent::UpdateOption()){
			flush_rewrite_rules(true);
			return true;
		}else{
		    return false;
		}
	}
	
	public static function GetAPIMessages($id='',$default='') {
	    $msg=[];
		$msg['m001'] = "Your host (%s) has been automatically blocked for anonymous call";
		$msg['m002'] = "Request param is invalid";
		$msg['m003'] = "Invalid license information";
		$msg['m004'] = "Invalid request, product information missing";
		$msg['m005'] = "Client has been in inactive status";
		$msg['m006'] = "Client has been deleted";
		$msg['m007'] = "Invalid license code";
		$msg['m008'] = "License successfully verified";
		$msg['m009'] = "Your purchase key has been refunded";
		$msg['m010'] = "Your purchase key has been expired";
		$msg['m011'] = "Your purchase key has been temporary inactivated";
		$msg['m012'] = "Your key has been installed on another application";
		$msg['m013'] = "Your key has been installed on another domain(%s)";
		$msg['m014'] = "Your purchase key has not been activated";
		$msg['m015'] = "Your key has been inactivated";
		//$msg['m016'] = "";
		$msg['m017'] = "You license key has been waiting for manual approval, Please contact with license author";
		$msg['m018'] = "License quota has been over, you can not add more domain with this license key";
		$msg['m019'] = "License deactivated successfully";
		$msg['m020'] = "Your purchase key is in different status, please contact with author";
		$msg['m021'] = "You key deactivate process has been failed. Please try again";
		$api_message=get_option('elite_api_msg',[]);
		if(!empty($api_message) && is_array($api_message)){
		    foreach ($api_message as $key=>$value){
			    $msg[$key]=$value;
		    }
		}
		if(!empty($id)){
		    if(!empty($msg[$id])){
		        return $msg[$id];
		    }
		    return $default;
		}
		return $msg;
	}
	
}